@@ -7,7 +7,7 @@ import os
 import requests
 
 from .base import GenericAPI, ERROR_THRESHOLD
-from ..metadata import MetadataItem
+from ..metadata import MetadataItem, to_api_format
 from .. import utils
 
 
@@ -93,7 +93,7 @@ class FilesAPI(GenericAPI):
 
     Parameters
     ----------
-    auth : auth.Authorization
+    auth : hyperthought.auth Authentication class
         Authorization object used to get headers and cookies needed to call
         HyperThought endpoints.
     """
@@ -650,7 +650,7 @@ class FilesAPI(GenericAPI):
         )
 
         # Use the url to upload the file.
-        self._upload_using_url(
+        self.upload_using_url(
             url,
             active_local_path,
             progress_callback,
@@ -772,6 +772,203 @@ class FilesAPI(GenericAPI):
             # NOTE:  This method throws exceptions.
             self._report_api_error(r)
 
+    def bulk_metadata_update(self, file_ids, new_metadata):
+        """
+        Updates metadata for a group of files.
+
+        Parameters
+        -------------------------------------
+        file_ids : str
+            List of the unique file id numbers for the files that are
+            being updated.
+
+        new_metadata : list of MetadataItem or None
+            New metadata being written to the file. Will replace any
+            existing metadata. Merging will need to be done client
+            side.
+        """
+        # Validate input parameters
+        new_metadata = self._validate_metadata(new_metadata)
+        api_formatted_metadata = to_api_format(metadata=new_metadata)
+
+        for id_ in file_ids:
+            self._validate_id(id_)
+
+        curried_request = partial(
+            requests.patch,
+            url=f"{self._base_url}/api/files/bulk-update/",
+            json={
+                'fileIds': file_ids,
+                'newMetadata': api_formatted_metadata,
+            }
+        )
+
+        r = self.attempt_api_call(curried_request=curried_request)
+
+        if r.status_code >= ERROR_THRESHOLD:
+            # NOTE: This method throws exceptions.
+            self._report_api_error(r)
+
+    def upload_using_url(self, upload_url, local_path,
+                         progress_callback=None, n_chunks=100):
+        """
+        Use a url to upload a file.
+
+        Called from self.upload.
+
+        Parameters
+        ----------
+        upload_url : str
+            The url to which the file should be uploaded.
+        local_path : str
+            The local path to the file to be uploaded.
+        progress_callback : callable(int -> None) or None
+            A callable to provide progress on upload status.
+        n_chunks : int
+            The number of chunks to be handled by progress_callback.
+            Will be ignored if progress_callback is None.
+        """
+        upload_url = self._validate_url(upload_url)
+        local_path = self._validate_local_path(local_path)
+
+        file_size = os.path.getsize(local_path)
+        file_handle = open(local_path, 'rb')
+
+        if progress_callback is not None:
+            chunk_size = math.ceil(file_size / n_chunks)
+            progress_handler = FileTransferProgressHandler(
+                chunk_size=chunk_size,
+                total_size=file_size,
+                callback=progress_callback,
+            )
+            original_read = file_handle.read
+
+            def new_read(size):
+                progress_handler.add(size)
+                return original_read(size)
+
+            file_handle.read = new_read
+
+        # NOTE:  This request will not be curried and passed to
+        #        self.attempt_api_call, since it is not clear that multiple
+        #        attempts could be made with a presigned url.
+        kwargs = {
+            'url': upload_url,
+            'data': file_handle,
+            'verify': False,
+            'stream': True,
+            'headers': {},
+        }
+
+        if self.get_backend() == 'default':
+            kwargs['headers'].update(self._auth.get_headers())
+            # Content-Disposition (with file name) is required by Django 2.2.
+            sep = utils.PATH_SEP
+            file_name = local_path.strip(sep).split(sep)[-1]
+            kwargs['headers']['Content-Disposition'] = (
+                f"inline;filename={file_name}")
+            kwargs['cookies'] = self._auth.get_cookies()
+        else:
+            # # TODO:  Why can't this be removed?
+            kwargs['headers'].update({
+                'Content-Type': 'application/octet-stream'
+            })
+
+        r = requests.put(**kwargs)
+
+        if r.status_code >= ERROR_THRESHOLD:
+            self._report_api_error(response=r)
+
+        file_handle.close()
+
+    def generate_blank_upload_urls(self, url_count):
+        """
+        Generate blank upload urls.
+
+        Blank urls can be used with any file.
+
+        Parameters
+        ----------
+        url_count : int
+            The number of urls to generate.
+
+        Returns
+        -------
+        A dictionary (file key -> url).  See the create_permanent_files method
+        for instructions on how to use the file keys after the urls have been
+        used to upload the files.
+        """
+        if not isinstance(url_count, int) or url_count <= 0:
+            raise ValueError("url_count must be a positive integer")
+
+        curried_request = partial(
+            requests.post,
+            url=f"{self._files_base_url}generate-blank-upload-urls/",
+            json={
+                'urlCount': url_count
+            },
+        )
+        r = self.attempt_api_call(curried_request=curried_request)
+
+        if r.status_code >= ERROR_THRESHOLD:
+            # NOTE:  This method will throw an exception.
+            self._report_api_error(response=r)
+
+        urls = r.json()["urls"]
+        return urls
+
+    def create_files_after_upload(self, file_info):
+        """
+        Create file database documents after using blank urls to upload files.
+
+        Parameters
+        ----------
+        file_info : list of dict
+            Each element must contain the following keys:
+                id : str
+                    The id for the file/folder to be created.
+                    Must be pre-specified.
+                name : str
+                    The name of the file, including its extension.
+                type : str
+                    The type of the file system object.  Must be "file" or
+                    "folder".
+                spaceId : str
+                    The id of the workspace of interest.
+                path : str
+                    Comma-separated list of ancester folder ids,
+                    e.g. ",UUID,UUID,UUID,"
+                size : int
+                    Size of the file in bytes.
+                    Only required for files.  Will be ignored for folders.
+                key : str
+                    The key to the file in the file storage backend.
+                    Only required for files.  Will be ignored for folders.
+            In addition, the following key may be provided:
+                metadata : list of dict
+                    API-formatted metadata to associate with the file.
+                    See the metadata module for more information.
+        """
+        # Add backend to file info.
+        backend = self.get_backend()
+
+        for item in file_info:
+            item["backend"] = backend
+
+        # The endpoint will take care of validating file info.
+        curried_request = partial(
+            requests.post,
+            url=f"{self._files_base_url}create-files-after-upload/",
+            json={
+                'fileInfo': file_info,
+            },
+        )
+        r = self.attempt_api_call(curried_request=curried_request)
+
+        if r.status_code >= ERROR_THRESHOLD:
+            # NOTE:  This method will throw an exception.
+            self._report_api_error(response=r)
+
     def get_backend(self):
         """
         Get the files backend.
@@ -879,78 +1076,6 @@ class FilesAPI(GenericAPI):
 
         return url, file_id
 
-    def _upload_using_url(self, upload_url, local_path,
-                          progress_callback, n_chunks):
-        """
-        Use a url to upload a file.
-
-        Called from self.upload.
-
-        Parameters
-        ----------
-        upload_url : str
-            The url to which the file should be uploaded.
-        local_path : str
-            The local path to the file to be uploaded.
-        progress_callback : callable(int -> None) or None
-            A callable to provide progress on upload status.
-        n_chunks : int
-            The number of chunks to be handled by progress_callback.
-            Will be ignored if progress_callback is None.
-        """
-        upload_url = self._validate_url(upload_url)
-        local_path = self._validate_local_path(local_path)
-
-        file_size = os.path.getsize(local_path)
-        file_handle = open(local_path, 'rb')
-
-        if progress_callback is not None:
-            chunk_size = math.ceil(file_size / n_chunks)
-            progress_handler = FileTransferProgressHandler(
-                chunk_size=chunk_size,
-                total_size=file_size,
-                callback=progress_callback,
-            )
-            original_read = file_handle.read
-
-            def new_read(size):
-                progress_handler.add(size)
-                return original_read(size)
-
-            file_handle.read = new_read
-
-        # NOTE:  This request will not be curried and passed to
-        #        self.attempt_api_call, since it is not clear that multiple
-        #        attempts could be made with a presigned url.
-        kwargs = {
-            'url': upload_url,
-            'data': file_handle,
-            'verify': False,
-            'stream': True,
-            'headers': {},
-        }
-
-        if self.get_backend() == 'default':
-            kwargs['headers'].update(self._auth.get_headers())
-            # Content-Disposition (with file name) is required by Django 2.2.
-            sep = utils.PATH_SEP
-            file_name = local_path.strip(sep).split(sep)[-1]
-            kwargs['headers']['Content-Disposition'] = (
-                f"inline;filename={file_name}")
-            kwargs['cookies'] = self._auth.get_cookies()
-        else:
-            # TODO:  Why can't this be removed?
-            kwargs['headers'].update({
-                'Content-Type': 'application/octet-stream'
-            })
-
-        r = requests.put(**kwargs)
-
-        if r.status_code >= ERROR_THRESHOLD:
-            self._report_api_error(response=r)
-
-        file_handle.close()
-
     def _temp_to_perm(self, file_id):
         """
         Move a file from the temporary (invisible) to the permanent (visible)
